// SPDX-License-Identifier: GPL-2.0-only
/*
 * Driver for the Epson RTC modules RA4000CE and RA8000CE
 * Copyright(C) SEIKO EPSON CORPORATION 2022. All rights reserved.
 *
 * Based on code from drivers/rtc/rtc-rx6110.c
 * Copyright(C) 2015 Pengutronix, Steffen Trumtrar <kernel@pengutronix.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#if 1				/* Set to 1 to enable debugging */
#define DEBUG
#include <linux/device.h>
#undef DEBUG
//#define EXTRA_DEBUGGING		/* Define for even more debugging */
//#define DUMP_REGISTERS		/* Define to dump RTC registers */
#endif

#include <linux/bcd.h>
#include <linux/i2c.h>
#include <linux/spi/spi.h>
#include <linux/module.h>
#include <linux/of_device.h>
#include <linux/rtc.h>
#include <linux/regmap.h>

/* RA4000/RA8000 Register definitions */
#define RA48000_REG_SEC		0x00
#define RA48000_REG_MIN		0x01
#define RA48000_REG_HOUR	0x02
#define RA48000_REG_WEEKDAY	0x03
#define RA48000_REG_DAY		0x04
#define RA48000_REG_MONTH	0x05
#define RA48000_REG_YEAR	0x06
#define RA48000_REG_ALM_MIN	0x07
#define RA48000_REG_ALM_HOUR	0x08
#define RA48000_REG_ALM_WEEKDAY	0x09
#define RA48000_REG_WTCNT_L	0x0A
#define RA48000_REG_WTCNT_M	0x0B
#define RA48000_REG_WTCNT_H	0x0C
#define RA48000_REG_TCTL	0x0D
#define RA48000_REG_INTF	0x0E
#define RA48000_REG_INTE	0x0F

#define RA48000_REG_CNTSEL	0x1F
#define RA48000_REG_BUF1_CFG2	0x29
#define RA48000_REG_UPDISEL	0x34

#define RA48000_REG_SRAM00	0x60
#define RA48000_REG_SRAM01	0x61
#define RA48000_REG_SRAM02	0x62
#define RA48000_REG_SRAM03	0x63
#define RA48000_REG_SRAM04	0x64
#define RA48000_REG_SRAM05	0x65
#define RA48000_REG_SRAM06	0x66
#define RA48000_REG_SRAM07	0x67
#define RA48000_REG_SRAM08	0x68
#define RA48000_REG_SRAM09	0x69
#define RA48000_REG_SRAM0A	0x6A
#define RA48000_REG_SRAM0B	0x6B
#define RA48000_REG_SRAM0C	0x6C
#define RA48000_REG_SRAM0D	0x6D
#define RA48000_REG_SRAM0E	0x6E
#define RA48000_REG_SRAM0F	0x6F
#define RA48000_REG_SRAM10	0x70
#define RA48000_REG_SRAM11	0x71
#define RA48000_REG_SRAM12	0x72
#define RA48000_REG_SRAM13	0x73
#define RA48000_REG_SRAM14	0x74
#define RA48000_REG_SRAM15	0x75
#define RA48000_REG_SRAM16	0x76
#define RA48000_REG_SRAM17	0x77
#define RA48000_REG_SRAM18	0x78
#define RA48000_REG_SRAM19	0x79
#define RA48000_REG_SRAM1A	0x7A
#define RA48000_REG_SRAM1B	0x7B
#define RA48000_REG_SRAM1C	0x7C
#define RA48000_REG_SRAM1D	0x7D
#define RA48000_REG_SRAM1E	0x7E
#define RA48000_REG_SRAM1F	0x7F

/* Timer Control Register (0Dh) bit positions */
#define RA48000_BITS_TCTL_TSEL_1024HZ	0x00
#define RA48000_BITS_TCTL_TSEL_64HZ	0x01
#define RA48000_BITS_TCTL_TSEL_1HZ	0x02
#define RA48000_BITS_TCTL_TSEL_MIN	0x03
#define RA48000_BIT_TCTL_WADA		BIT(3)
#define RA48000_BIT_TCTL_TE		BIT(4)
#define RA48000_BIT_TCTL_USEL0		BIT(5)
#define RA48000_BITS_TCTL_FSEL_32768HZ	0x00
#define RA48000_BITS_TCTL_FSEL_1024HZ	0x40
#define RA48000_BITS_TCTL_FSEL_1HZ	0x80
#define RA48000_BITS_TCTL_FSEL_OFF	0xC0

/* Status Flag Register (0Eh) bit positions */
#define RA48000_BIT_INTF_VTMPLF		BIT(0)
#define RA48000_BIT_INTF_VLF		BIT(1)
#define RA48000_BIT_INTF_EVF		BIT(2)
#define RA48000_BIT_INTF_AF		BIT(3)
#define RA48000_BIT_INTF_TF		BIT(4)
#define RA48000_BIT_INTF_UF		BIT(5)
#define RA48000_BIT_INTF_OSCSTPF	BIT(6)
#define RA48000_BIT_INTF_PORF		BIT(7)

/* Interrupt Enable (0Fh) bit positions */
#define RA48000_BIT_INTE_STOP		BIT(0)
#define RA48000_BIT_INTE_EIE		BIT(2)
#define RA48000_BIT_INTE_AIE		BIT(3)
#define RA48000_BIT_INTE_TIE		BIT(4)
#define RA48000_BIT_INTE_UIE		BIT(5)
#define RA48000_BIT_INTE_CSEL_500MS	0x00
#define RA48000_BIT_INTE_CSEL_2SEC	0x40	/* default */
#define RA48000_BIT_INTE_CSEL_10SEC	0x80
#define RA48000_BIT_INTE_CSEL_30SEC	0xC0

/* Miscellaneous bit positions */
#define RA48000_BIT_ALARM_XWAE		BIT(7)
#define RA48000_BIT_CNTSEL_BCDCNTEN	BIT(0)
#define RA48000_BIT_CNTSEL_BINCNTEN	BIT(1)
#define RA48000_BIT_CNTSEL_ACCBCD	BIT(2)
#define RA48000_BIT_B1C2_SRAMMOD	BIT(6)
#define RA48000_BIT_UPDISEL_USEL1	BIT(1)

enum {
	RTC_SEC = 0,
	RTC_MIN,
	RTC_HOUR,
	RTC_WDAY,
	RTC_DAY,
	RTC_MONTH,
	RTC_YEAR,
	RTC_TIME_SIZE
};

#define RA4000_DRIVER_NAME		"ra4000ce"
#define RA8000_DRIVER_NAME		"ra8000ce"

struct ra48000_data {
	struct device *dev;
	struct rtc_device *rtc;
	struct regmap *regmap;
	int irq;
};

static int ra48000_get_time(struct device *dev, struct rtc_time *tm)
{
	struct ra48000_data *ra48000 = dev_get_drvdata(dev);
	int ret;
	int flags;
	u8 data[RTC_TIME_SIZE];

	ret = regmap_read(ra48000->regmap, RA48000_REG_INTF, &flags);
	if (ret) {
		dev_err(dev, "%s: read error %d\n", __func__, ret);
		return ret;
	}

	/* check for VLF Flag (set at power-on) */
	if ((flags & RA48000_BIT_INTF_VLF)) {
		dev_warn_once(dev, "Voltage was low, data is invalid!\n");
		return -EINVAL;
	}

	/* read registers to date */
	ret = regmap_bulk_read(ra48000->regmap, RA48000_REG_SEC, data, ARRAY_SIZE(data));
	if (ret) {
		dev_err(dev, "%s: read error %d\n", __func__, ret);
		return ret;
	}

	tm->tm_sec  = bcd2bin(data[RTC_SEC] & 0x7F);
	tm->tm_min  = bcd2bin(data[RTC_MIN] & 0x7F);
	tm->tm_hour = bcd2bin(data[RTC_HOUR] & 0x3F);		/* only 24-hour clock */
	tm->tm_wday = ffs(data[RTC_WDAY] & 0x7F) - 1;
	tm->tm_mday = bcd2bin(data[RTC_DAY] & 0x3F);
	tm->tm_mon  = bcd2bin(data[RTC_MONTH] & 0x1F) - 1;
	tm->tm_year = bcd2bin(data[RTC_YEAR]) + 100;

	dev_dbg(dev, "%s: sec=%d min=%d hour=%d mday=%d mon=%d year=%d wday=%d\n",
		__func__, tm->tm_sec, tm->tm_min, tm->tm_hour, tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900, tm->tm_wday);

	return 0;
}

static int ra48000_set_time(struct device *dev, struct rtc_time *tm)
{
	struct ra48000_data *ra48000 = dev_get_drvdata(dev);
	int ret;
	u8 data[RTC_TIME_SIZE];

	dev_dbg(dev, "%s: sec=%d min=%d hour=%d mday=%d mon=%d year=%d wday=%d\n",
		__func__, tm->tm_sec, tm->tm_min, tm->tm_hour, tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900, tm->tm_wday);

	data[RTC_SEC]   = bin2bcd(tm->tm_sec);
	data[RTC_MIN]   = bin2bcd(tm->tm_min);
	data[RTC_HOUR]  = bin2bcd(tm->tm_hour);
	data[RTC_WDAY]  = BIT(tm->tm_wday);
	data[RTC_DAY]   = bin2bcd(tm->tm_mday);
	data[RTC_MONTH] = bin2bcd(tm->tm_mon + 1);
	data[RTC_YEAR]  = bin2bcd(tm->tm_year - 100);

	/* set STOP bit before changing clock/calendar */
	ret = regmap_set_bits(ra48000->regmap, RA48000_REG_INTE, RA48000_BIT_INTE_STOP);
	if (ret)
		return ret;

	ret = regmap_bulk_write(ra48000->regmap, RA48000_REG_SEC, data, ARRAY_SIZE(data));
	if (ret) {
		dev_err(dev, "%s: write error %d\n", __func__, ret);
		return ret;
	}

	/* clear STOP bit after changing clock/calendar */
	ret = regmap_clear_bits(ra48000->regmap, RA48000_REG_INTE, RA48000_BIT_INTE_STOP);
	if (ret)
		return ret;

	/* The time in the RTC is valid. Be sure to have VLF cleared. */
	ret = regmap_write(ra48000->regmap, RA48000_REG_INTF, ~(u8)RA48000_BIT_INTF_VLF);
	if (ret) {
		dev_err(dev, "%s: write error %d\n", __func__, ret);
		return ret;
	}

	return 0;
}

static irqreturn_t ra48000_irq(int irq, void *data)
{
	struct ra48000_data	*ra48000 = data;
	struct mutex		*lock = &ra48000->rtc->ops_lock;
	int flags;
	int ret;

	mutex_lock(lock);

	ret = regmap_read(ra48000->regmap, RA48000_REG_INTF, &flags);
	if (ret)
		goto out;

	dev_dbg(ra48000->dev, "%s: flags=%02X\n", __func__, flags);

	if (flags & RA48000_BIT_INTF_TF) {
		/* periodic */
		flags &= ~RA48000_BIT_INTF_TF;
		rtc_update_irq(ra48000->rtc, 1, RTC_PF | RTC_IRQF);
	}

	if (flags & RA48000_BIT_INTF_AF) {
		/* alarm */
		flags &= ~RA48000_BIT_INTF_AF;
		rtc_update_irq(ra48000->rtc, 1, RTC_AF | RTC_IRQF);
	}

	if (flags & RA48000_BIT_INTF_UF) {
		/* time update */
		flags &= ~RA48000_BIT_INTF_UF;
		rtc_update_irq(ra48000->rtc, 1, RTC_UF | RTC_IRQF);
	}

	regmap_write(ra48000->regmap, RA48000_REG_INTF, flags);
out:
	mutex_unlock(lock);

	return IRQ_HANDLED;
}

static int ra48000_read_alarm(struct device *dev, struct rtc_wkalrm *alrm)
{
	struct ra48000_data *ra48000 = dev_get_drvdata(dev);
	u8 data[3], flct[2];
	int ret;

	/* Read alarm registers. */
	ret = regmap_bulk_read(ra48000->regmap, RA48000_REG_ALM_MIN, data, ARRAY_SIZE(data));
	if (ret)
		return ret;
	dev_dbg(dev, "%s: alarm read %02X %02X %02X\n", __func__, data[0], data[1], data[2]);

	/* Read flag and control registers. */
	ret = regmap_bulk_read(ra48000->regmap, RA48000_REG_INTF, flct, ARRAY_SIZE(flct));
	if (ret)
		return ret;

	/* Report alarm 0 time assuming 24-hour and day-of-month modes. */
	alrm->time.tm_sec  = 0;
	alrm->time.tm_min  = bcd2bin(data[0] & 0x7F);
	alrm->time.tm_hour = bcd2bin(data[1] & 0x3F);
	if (!(data[2] & RA48000_BIT_ALARM_XWAE))
		alrm->time.tm_mday = bcd2bin(data[2] & 0x3F);

	alrm->enabled = !!(flct[1] & RA48000_BIT_INTE_AIE);
	alrm->pending = (flct[0] & RA48000_BIT_INTF_AF) && alrm->enabled;

	dev_dbg(dev, "%s: sec=%d min=%d hour=%d mday=%d mon=%d wday=%d enabled=%d pending=%d\n",
		__func__, alrm->time.tm_sec, alrm->time.tm_min, alrm->time.tm_hour, alrm->time.tm_mday, alrm->time.tm_mon+1, alrm->time.tm_wday, alrm->enabled, alrm->pending);

	return 0;
}

static int ra48000_set_alarm(struct device *dev, struct rtc_wkalrm *alrm)
{
	struct ra48000_data *ra48000 = dev_get_drvdata(dev);
	u8 data[3], flct[2];
	int ret;

#ifdef EXTRA_DEBUGGING
	dev_dbg(dev, "set_alarm %s\n", alrm->enabled?"YES":"NO");
	dev_dbg(dev, "set_alarm UIE %d\n", ra48000->rtc->uie_rtctimer.enabled );
	dev_dbg(dev, "set_alarm AIR %d\n", ra48000->rtc->aie_timer.enabled );
#endif

	dev_dbg(dev, "%s: sec=%d min=%d hour=%d mday=%d mon=%d wday=%d enabled=%d pending=%d\n",
		__func__, alrm->time.tm_sec, alrm->time.tm_min, alrm->time.tm_hour, alrm->time.tm_mday, alrm->time.tm_mon+1, alrm->time.tm_wday, alrm->enabled, alrm->pending);

	/* Read flag and control registers. */
	ret = regmap_bulk_read(ra48000->regmap, RA48000_REG_INTF, flct, ARRAY_SIZE(flct));
	if (ret)
		return ret;

	if (flct[1] & (RA48000_BIT_INTE_AIE | RA48000_BIT_INTE_UIE)) {
		flct[1] &= ~(RA48000_BIT_INTE_AIE | RA48000_BIT_INTE_UIE);
		ret = regmap_write(ra48000->regmap, RA48000_REG_INTE, flct[1]);
		if (ret)
			return ret;
	}

	/* Hardware alarm precision is 1 minute! */
	data[0] = bin2bcd(alrm->time.tm_min);
	data[1] = bin2bcd(alrm->time.tm_hour);
	data[2] = bin2bcd(alrm->time.tm_mday);
	if (data[2] == 0)
		data[2] |= RA48000_BIT_ALARM_XWAE;		/* turn on XWAE bit to ignore day of month (no zero day) */
	dev_dbg(dev, "%s: alarm write %02X %02X %02X\n", __func__, data[0], data[1], data[2]);

	ret = regmap_bulk_write(ra48000->regmap, RA48000_REG_ALM_MIN, data, ARRAY_SIZE(data));
	if (ret)
		return ret;

	flct[0] &= ~(RA48000_BIT_INTF_AF | RA48000_BIT_INTF_UF);
	ret = regmap_write(ra48000->regmap, RA48000_REG_INTF, flct[0]);
	if (ret)
		return ret;

	if (!alrm->enabled)
		return 0;

	if ( ra48000->rtc->uie_rtctimer.enabled )
		flct[1] |= RA48000_BIT_INTE_UIE;					/* set update interrupt enable */
	if ( ra48000->rtc->aie_timer.enabled )
		flct[1] |= (RA48000_BIT_INTE_AIE | RA48000_BIT_INTE_UIE);		/* set alarm interrupt enable */

	return regmap_write(ra48000->regmap, RA48000_REG_INTE, flct[1]);
}

static int ra48000_alarm_irq_enable(struct device *dev, unsigned int enabled)
{
	struct ra48000_data *ra48000 = dev_get_drvdata(dev);
	int ctrl;
	int ret;

#ifdef EXTRA_DEBUGGING
	dev_dbg(dev, "alarm_irq_enable %s\n", enabled?"ON":"OFF");
	dev_dbg(dev, "alarm_irq_enable UIE %d\n", ra48000->rtc->uie_rtctimer.enabled );
	dev_dbg(dev, "alarm_irq_enable AIR %d\n", ra48000->rtc->aie_timer.enabled );
#endif

	ret = regmap_read(ra48000->regmap, RA48000_REG_INTE, &ctrl);
	if (ret)
		return ret;

	if (enabled) {
		if ( ra48000->rtc->uie_rtctimer.enabled )
			ctrl |= RA48000_BIT_INTE_UIE;				/* set update interrupt enable */
		if ( ra48000->rtc->aie_timer.enabled )
			ctrl |= (RA48000_BIT_INTE_AIE | RA48000_BIT_INTE_UIE);	/* set alarm interrupt enable */
	} else {
		if ( !ra48000->rtc->uie_rtctimer.enabled )
			ctrl &= ~RA48000_BIT_INTE_UIE;				/* clear update interrupt enable */
		if ( !ra48000->rtc->aie_timer.enabled )
			ctrl &= ~RA48000_BIT_INTE_AIE;				/* clear alarm interrupt enable */
	}

	ret = regmap_clear_bits(ra48000->regmap, RA48000_REG_INTF, RA48000_BIT_INTF_AF);
	if (ret)
		return ret;

	return regmap_write(ra48000->regmap, RA48000_REG_INTE, ctrl);
}

static int ra48000_nvmem_read(void *priv, unsigned int offset, void *val, size_t bytes)
{
	u8 *data = val;
	size_t read;
	int ret;

	ret = regmap_write(priv, RA48000_REG_BUF1_CFG2, RA48000_BIT_B1C2_SRAMMOD);
	if (ret)
		return ret;

	while (bytes) {
		read = 16 - (offset&0x0F);	/* limit read to this bank */
		if (read > bytes)
			read = bytes;
		ret = regmap_bulk_read(priv, RA48000_REG_SRAM00 + offset, data, read);
		if (ret)
			return ret;
		data += read;
		offset += read;
		bytes -= read;
	}

	return 0;
}

static int ra48000_nvmem_write(void *priv, unsigned int offset, void *val, size_t bytes)
{
	u8 *data = val;
	size_t write;
	int ret;

	ret = regmap_write(priv, RA48000_REG_BUF1_CFG2, RA48000_BIT_B1C2_SRAMMOD);
	if (ret)
		return ret;

	while (bytes) {
		write = 16 - (offset&0x0F);	/* limit write to this bank */
		if (write > bytes)
			write = bytes;
		ret = regmap_bulk_write(priv, RA48000_REG_SRAM00 + offset, data, write);
		if (ret)
			return ret;
		data += write;
		offset += write;
		bytes -= write;
	}

	return 0;
}

static const struct rtc_class_ops ra48000_rtc_ops = {
	.read_time	= ra48000_get_time,
	.set_time	= ra48000_set_time,
	.read_alarm	= ra48000_read_alarm,
	.set_alarm	= ra48000_set_alarm,
	.alarm_irq_enable = ra48000_alarm_irq_enable,
};

#ifdef DUMP_REGISTERS
static void ra48000_dump_bank(struct device *dev, int bank, u8 *data, int len)
{
	int i;
	int n;
	char out[128];

	n = sprintf(out, "Bank%d:", bank);
	for (i = 0; i < len; i++)
		n += sprintf(out+n, " %02X", data[i]);
	dev_dbg(dev, "%s\n", out);
}

static void ra48000_dump_registers(struct ra48000_data *ra48000)
{
	int ret;
	int bank;
	int len;
	u8 data[16];

	for (bank = 0; bank < 8; bank++) {
		switch (bank) {
		case 0:
			len = 16;
			ret = regmap_bulk_read(ra48000->regmap, 0x00, data, len);
			break;
		case 1:
			len = 16;
			ret = regmap_bulk_read(ra48000->regmap, 0x10, data, len);
			break;
		case 2:
			len = 5 + 3;
			ret = regmap_bulk_read(ra48000->regmap, 0x20, data, 5);
			if (!ret) ret = regmap_bulk_read(ra48000->regmap, 0x27, data+5, 3);
			break;
		case 3:
			len = 5 + 8;
			ret = regmap_bulk_read(ra48000->regmap, 0x30, data, 5);
			if (!ret) ret = regmap_bulk_read(ra48000->regmap, 0x38, data+5, 8);
			break;
		case 4:
			len = 7;
			ret = regmap_bulk_read(ra48000->regmap, 0x41, data, len);
			break;
		case 5:
			len = 4;
			ret = regmap_bulk_read(ra48000->regmap, 0x51, data, len);
			break;
		case 6:
			len = 16;
			ret = regmap_bulk_read(ra48000->regmap, 0x60, data, len);
			break;
		case 7:
			len = 16;
			ret = regmap_bulk_read(ra48000->regmap, 0x70, data, len);
			break;
		}
		if (!ret)
			ra48000_dump_bank(ra48000->dev, bank, data, len);
	}
}
#else
  #define ra48000_dump_registers(r)
#endif

static int ra48000_init(struct ra48000_data *ra48000)
{
	struct device *dev = ra48000->dev;
	int flags;
	int ret;
	static const struct reg_sequence ra48000_default_regs[] = {
		{ RA48000_REG_CNTSEL,		RA48000_BIT_CNTSEL_BCDCNTEN | RA48000_BIT_CNTSEL_ACCBCD },	/* BINCNTEN=0 */
		{ RA48000_REG_ALM_MIN,		0x00 },
		{ RA48000_REG_ALM_HOUR,		0x00 },
		{ RA48000_REG_ALM_WEEKDAY,	0x00 },
		{ RA48000_REG_UPDISEL,		0x00 },		/* USEL1=0 */
		{ RA48000_REG_TCTL,		RA48000_BITS_TCTL_TSEL_1HZ | RA48000_BIT_TCTL_WADA | RA48000_BITS_TCTL_FSEL_OFF },	/* USEL0=0, TE=0 */
		{ RA48000_REG_BUF1_CFG2,	RA48000_BIT_B1C2_SRAMMOD },
	};

	ra48000_dump_registers(ra48000);

	ret = regmap_register_patch(ra48000->regmap, ra48000_default_regs, ARRAY_SIZE(ra48000_default_regs));
	if (ret) {
		dev_err(dev, "%s: patch write error %d\n", __func__, ret);
		return ret;
	}

	ret = regmap_read(ra48000->regmap, RA48000_REG_INTF, &flags);
	if (ret) {
		dev_err(dev, "%s: flag read error %d\n", __func__, ret);
		return ret;
	}

#ifdef EXTRA_DEBUGGING
	if (flags & RA48000_BIT_INTF_PORF)
		dev_warn(dev, "Power on reset was detected.\n");
	if (flags & RA48000_BIT_INTF_OSCSTPF)
		dev_warn(dev, "Oscillation stop was detected.\n");
	if ((flags & RA48000_BIT_INTF_VTMPLF))
		dev_warn(dev, "Voltage drop was detected.\n");
#endif

	/* clear all flags BUT VLF */
	flags &= ~(RA48000_BIT_INTF_VTMPLF | RA48000_BIT_INTF_EVF | RA48000_BIT_INTF_AF | RA48000_BIT_INTF_TF | RA48000_BIT_INTF_UF | RA48000_BIT_INTF_OSCSTPF | RA48000_BIT_INTF_PORF);
	ret = regmap_write(ra48000->regmap, RA48000_REG_INTF, flags);
	if (ret) {
		dev_err(dev, "%s: flag write error %d\n", __func__, ret);
		return ret;
	}

	return ret;
}

static int ra48000_probe(struct ra48000_data *ra48000, struct device *dev)
{
	int err;
	struct nvmem_config nvmem_cfg = {
		.name = "ra48000ce_nvmem",
		.word_size = 1,
		.stride = 1,
		.size = (RA48000_REG_SRAM1F - RA48000_REG_SRAM00 + 1),
		.reg_read = ra48000_nvmem_read,
		.reg_write = ra48000_nvmem_write,
		.type = NVMEM_TYPE_BATTERY_BACKED,
	};

	ra48000->rtc = devm_rtc_allocate_device(dev);
	if (IS_ERR(ra48000->rtc)) {
		dev_err(dev, "%s: rtc device allocation failed!\n", __func__);
		return PTR_ERR(ra48000->rtc);
	}

	err = ra48000_init(ra48000);
	if (err)
		return err;

	if (ra48000->irq > 0) {
		err = devm_request_threaded_irq(dev, ra48000->irq, NULL, ra48000_irq, IRQF_SHARED | IRQF_ONESHOT, dev_name(dev), ra48000);
		if (err) {
			ra48000->irq = 0;
			device_set_wakeup_capable(dev, false);
			dev_warn(dev, "%s: unable to request IRQ!\n", __func__);
		} else {
			device_set_wakeup_capable(dev, true);
			dev_dbg(dev, "%s: IRQ=%d\n", __func__, ra48000->irq);
		}
	}

	ra48000->rtc->ops = &ra48000_rtc_ops;
	ra48000->rtc->range_min = RTC_TIMESTAMP_BEGIN_2000;
	ra48000->rtc->range_max = RTC_TIMESTAMP_END_2099;
	ra48000->rtc->max_user_freq = 1;

	err = devm_rtc_register_device(ra48000->rtc);
	if (err) {
		dev_err(dev, "%s: rtc device registration failed!\n", __func__);
		return err;
	}

	nvmem_cfg.priv = ra48000->regmap;
	devm_rtc_nvmem_register(ra48000->rtc, &nvmem_cfg);

	return 0;
}

#ifdef CONFIG_OF
static const struct of_device_id ra48000_of_match[] = {
	{ .compatible = "epson,ra4000ce" },
	{ .compatible = "epson,ra8000ce" },
	{ },
};
MODULE_DEVICE_TABLE(of, ra48000_of_match);
#endif

#if IS_ENABLED(CONFIG_SPI_MASTER)
static struct regmap_config ra4000_regmap_config = {
	.reg_bits = 8,
	.val_bits = 8,
	.max_register = RA48000_REG_SRAM1F,
	.read_flag_mask = 0x80,
};

static int ra4000_probe(struct spi_device *spi)
{
	struct device		*dev = &spi->dev;
	struct ra48000_data	*ra48000;
	int			err;

	spi->mode |= SPI_MODE_3;
/*	spi->mode ^= SPI_CS_HIGH; */	/* CS active high must be set in device tree! */
	spi->bits_per_word = 8;
	spi->max_speed_hz = 2000000;
	err = spi_setup(spi);
	if (err) {
		dev_err(dev, "%s: SPI setup failed!\n", __func__);
		return err;
	}

	dev_dbg(dev, "SPI settings: bits_per_word=%d, max_speed_hz=%d, mode=%Xh\n",
		 spi->bits_per_word, spi->max_speed_hz, spi->mode);

	ra48000 = devm_kzalloc(dev, sizeof(*ra48000), GFP_KERNEL);
	if (!ra48000)
		return -ENOMEM;

	ra48000->dev = dev;
	ra48000->regmap = devm_regmap_init_spi(spi, &ra4000_regmap_config);
	if (IS_ERR(ra48000->regmap)) {
		dev_err(dev, "%s: regmap init failed!\n", __func__);
		return PTR_ERR(ra48000->regmap);
	}

	ra48000->irq = spi->irq;
	spi_set_drvdata(spi, ra48000);

	return ra48000_probe(ra48000, dev);
}

static const struct spi_device_id ra4000_id[] = {
	{ "ra4000ce", 0 },
/*	{ "ra8000ce", 1 }, */
	{ }
};
MODULE_DEVICE_TABLE(spi, ra4000_id);

static struct spi_driver ra4000_driver = {
	.driver = {
		.name = RA4000_DRIVER_NAME,
		.of_match_table = of_match_ptr(ra48000_of_match),
	},
	.probe		= ra4000_probe,
	.id_table	= ra4000_id,
};

static int ra4000_register_driver(void)
{
	return spi_register_driver(&ra4000_driver);
}

static void ra4000_unregister_driver(void)
{
	spi_unregister_driver(&ra4000_driver);
}
#else
static int ra4000_register_driver(void)
{
	return 0;
}

static void ra4000_unregister_driver(void)
{
}
#endif /* CONFIG_SPI_MASTER */

#if IS_ENABLED(CONFIG_I2C)
static struct regmap_config ra8000_regmap_config = {
	.reg_bits = 8,
	.val_bits = 8,
	.max_register = RA48000_REG_SRAM1F,
	.read_flag_mask = 0x80,
};

static int ra8000_probe(struct i2c_client *client)
{
	struct device		*dev = &client->dev;
	struct ra48000_data	*ra48000;

	if (!i2c_check_functionality(client->adapter, I2C_FUNC_I2C)) {
		dev_err(&client->adapter->dev, "%s: Adapter doesn't support required I2C functionality!\n", __func__);
		return -ENODEV;
	}

	ra48000 = devm_kzalloc(dev, sizeof(*ra48000), GFP_KERNEL);
	if (!ra48000)
		return -ENOMEM;

	ra48000->dev = dev;
	ra48000->regmap = devm_regmap_init_i2c(client, &ra8000_regmap_config);
	if (IS_ERR(ra48000->regmap)) {
		dev_err(dev, "%s: regmap init failed!\n", __func__);
		return PTR_ERR(ra48000->regmap);
	}

	ra48000->irq = client->irq;
	i2c_set_clientdata(client, ra48000);

	return ra48000_probe(ra48000, dev);
}

static const struct i2c_device_id ra8000_id[] = {
	{ "ra8000ce", 0 },
	{ }
};
MODULE_DEVICE_TABLE(i2c, ra8000_id);

static struct i2c_driver ra8000_driver = {
	.driver = {
		.name = RA8000_DRIVER_NAME,
		.of_match_table = of_match_ptr(ra48000_of_match),
	},
	.probe_new	= ra8000_probe,
	.id_table	= ra8000_id,
};

static int ra8000_register_driver(void)
{
	return i2c_add_driver(&ra8000_driver);
}

static void ra8000_unregister_driver(void)
{
	i2c_del_driver(&ra8000_driver);
}
#else
static int ra8000_register_driver(void)
{
	return 0;
}

static void ra8000_unregister_driver(void)
{
}
#endif /* CONFIG_I2C */

static int __init ra48000_module_init(void)
{
	int ret;

	ret = ra4000_register_driver();
	if (ret)
		return ret;

	ret = ra8000_register_driver();
	if (ret)
		ra4000_unregister_driver();

	return ret;
}
module_init(ra48000_module_init);

static void __exit ra48000_module_exit(void)
{
	ra4000_unregister_driver();
	ra8000_unregister_driver();
}
module_exit(ra48000_module_exit);

MODULE_AUTHOR("Epson VDC <vdctech_info@ea.epson.com>");
MODULE_DESCRIPTION("RA4000/RA8000 CE RTC driver");
MODULE_LICENSE("GPL");
